// Log to console (see http://plasmasturm.org/log/186/)
function _rel_vcs_log_console(msg) {
    var debug_flag = false;

    if (debug_flag) {
	Components
	    .classes["@mozilla.org/consoleservice;1" ]
	    .getService(Components.interfaces["nsIConsoleService"])
	    .logStringMessage(msg);
    }
}

function get_HTML_by_tag_name(tag) {
    // from https://developer.mozilla.org/en/Writing_JavaScript_for_XHTML
    if (content.document.documentElement.namespaceURI !== null) {  // tag soup
	return content.document.getElementsByTagName(tag);
    } else {  // xml document
	return content.document.getElementsByTagNameNS("http://www.w3.org/1999/xhtml", tag);
    }
}

// Find links to rel-vcs data
function get_rel_vcs() {
    var links = new Array();
    if (content === null) {
	return links;
    }

    var all_links = get_HTML_by_tag_name('link');
    var i = 0;
    for (i=0; i < all_links.length; i++) {
	var link = all_links[i];
	_rel_vcs_log_console("check "+link.rel+" ? "+String(link.rel.match("^vcs-")));
	if (link.rel.match("^vcs-")) {
	    links.push(link.cloneNode(false));
	}
    }

    var all_links = get_HTML_by_tag_name('a');
    for (i=0; i < all_links.length; i++) {
	var link = all_links[i];
	_rel_vcs_log_console("check "+link.rel+" ? "+String(link.rel.match("^vcs-")));
	if (link.rel.match("^vcs-")) {
	    // Todo, read in tag body if title attribute is missing
	    links.push(link.cloneNode(false));
	}
    }

    return links;
}

function detect_rel_vcs(event) {
    var has_rel_vcs = (get_rel_vcs().length > 0);
    rel_vcs_display_status(has_rel_vcs);
}

// Extension status
function rel_vcs_display_status(has_rel_vcs) {
    _rel_vcs_log_console("has rel-vcs? "+String(has_rel_vcs));
    var item = document.getElementById("rel-vcs-icon");
    if (has_rel_vcs) {
    	item.setAttribute("state", "on");
    } else {
    	item.setAttribute("state", "off");
    }
}

function rel_vcs_about_dialog() {
  window.openDialog("chrome://rel-vcs/content/about.xul", 
		    "rel-vcs-about-dialog", 
		    "centerscreen,chrome,modal,resizable");
}

function rel_vcs_exec_action(button) {
    var links = get_rel_vcs();
    rel_vcs_popup_dialog(button, links);
}

function rel_vcs_popup_dialog(button, links) {
    if (links.length == 0) {
    	return;
    }

    var popmenu = document.getElementById("rel-vcs-popmenu");
    var icons = {
	"logo": "chrome://rel-vcs/skin/logo.png"
    };

    // clean the popup
    while (popmenu.childNodes.length > 0) {
	popmenu.removeChild(popmenu.lastChild);
    }

    // add all new items
    for (key in links) {
	var link = links[key];
	var rel = link.rel;
	var uri = link.href;
	var title = link.title;
	var menuitem = document.createElement('menuitem');
	menuitem.setAttribute('label', title+' ('+uri+')');
	menuitem.setAttribute('tooltiptext', 'Click to view '+uri);
	menuitem.setAttribute('onclick', 'rel_vcs_open_uri("'+uri+'", "'+rel+'")');
	menuitem.setAttribute('class', 'menuitem-iconic ' + rel);
	menuitem.setAttribute('image', icons['logo']); // icons[rel]
	popmenu.appendChild(menuitem);
    }

    // NOTE: wait only for DOM to load before running check for rel-vcs content  
    popmenu.showPopup(button, -1, -1, 'popup', 'topright' , 'bottomright');
}

function rel_vcs_open_uri(uri, rel) {
    var newTab = getBrowser().addTab(uri);
    getBrowser().selectedTab = newTab; // change the focused tab, feel free to delete
}

function init() {
    _rel_vcs_log_console("init rel-vcs");
    // Listen for Page loads
    getBrowser().addEventListener("load", detect_rel_vcs, true);
    // Listen for Tab switches
    getBrowser().addEventListener("select", detect_rel_vcs, true);
}

init(); 
